//+------------------------------------------------------------------+
//|                                             PivotPointsAlert.mq4 |
//|                             Copyright  2008, forex.topcoders.eu |
//|                                        http://forex.topcoders.eu |
//+------------------------------------------------------------------+
#property copyright "Copyright  2008, forex.topcoders.eu"
#property link      "http://forex.topcoders.eu"

#property indicator_chart_window
#property indicator_buffers 7
#property indicator_color1  White
#property indicator_color2  MediumSlateBlue
#property indicator_color3  MediumSlateBlue
#property indicator_color4  Red
#property indicator_color5  Red
#property indicator_color6  Blue
#property indicator_color7  Blue

#define MINUTES_PER_DAY 1440

extern string note0        = "Limit number of history bars";
extern int    maxBars      = 1000;
extern string note1        = "Colored chart";
extern bool   colored      = true;
extern string note2        = "Send e-mail";
extern bool   email        = true;
extern string emailBody    = "Alarm!";
extern string emailSubject = "PivotPoints Alert";
extern string note3        = "Notify when price cross R1 or S1";
extern bool   notify1      = true;
extern string note4        = "Notify when price cross R2 or S2";
extern bool   notify2      = false;
extern string note5        = "Notify when price cross R3 or S3";
extern bool   notify3      = false;

double bufferPP[];
double bufferR1[];
double bufferS1[];
double bufferR2[];
double bufferS2[];
double bufferR3[];
double bufferS3[];

int objCounter = 0;

bool alarm1sent = false, alarm2sent = false, alarm3sent = false,
     alarm4sent = false, alarm5sent = false, alarm6sent = false;

datetime startTime;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   IndicatorShortName("PivotPointsAlert");

   SetIndexBuffer(0, bufferPP); SetIndexLabel(0, "PP");
   SetIndexBuffer(1, bufferR1); SetIndexLabel(1, "R1");
   SetIndexBuffer(2, bufferS1); SetIndexLabel(2, "S1");
   SetIndexBuffer(3, bufferR2); SetIndexLabel(3, "R2");
   SetIndexBuffer(4, bufferS2); SetIndexLabel(4, "S2");
   SetIndexBuffer(5, bufferR3); SetIndexLabel(5, "R3");
   SetIndexBuffer(6, bufferS3); SetIndexLabel(6, "S3");    
   
   startTime = TimeCurrent();

   return(0);
}
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
   // Delete all colored rectangles
   for(int i = 0; i < objCounter; i++)
   {
      ObjectDelete("R1" + DoubleToStr(i, 0));
      ObjectDelete("S1" + DoubleToStr(i, 0));
      ObjectDelete("R2" + DoubleToStr(i, 0));
      ObjectDelete("S2" + DoubleToStr(i, 0));
      ObjectDelete("R3" + DoubleToStr(i, 0));
      ObjectDelete("S3" + DoubleToStr(i, 0));
   }
   
   return(0);
}
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   int barsPerDay = MathFloor(MINUTES_PER_DAY / Period());
   int bars = Bars;

   if(bars < barsPerDay)
   {
      Alert("No history! Go to history center and download it");
      return(1);
   }   

   if(bars > maxBars + barsPerDay)
      bars = maxBars + barsPerDay;
      
   double PP = 0.0;
   double R1, R2, R3 = 0.0;
   double S1, S2, S3 = 0.0;
   double LO =  1.7 * MathExp( 308); // MAX_DOUBLE
   double HI = -1.7 * MathExp(-308); // MIN_DOUBLE
   
   for(int i = bars - IndicatorCounted() - 1; i >= 0; i--)
   {
      // Canculate High/Low
      if(High[i] > HI) HI = High[i];
      if(Low [i] < LO) LO = Low [i];
      
      // New day comes ...
      if(TimeDay(Time[i]) != TimeDay(Time[i + 1]))
      {
         // Canculate Pivot Points
         PP = (HI + LO + Close[i + 1]) / 3;
         
         // Canculate support and resistance
         R1 = 2 * PP - LO;
         S1 = 2 * PP - HI;
         //
         R2 = PP - S1 + R1;
         S2 = PP - R1 + S1;
         // Initialize High/Low for a new day
         HI = High[i];
         LO = Low [i];
         //
		   R3 = R1 + (HI - LO);
			S3 = S1 - (HI - LO);
			
			// Draw colored rectangles
			if(colored)
			{
            ObjectCreate("R1" + DoubleToStr(objCounter, 0), OBJ_RECTANGLE, 0, Time[i], PP, Time[i], R1); 
            ObjectSet   ("R1" + DoubleToStr(objCounter, 0), OBJPROP_COLOR, MediumSlateBlue); objCounter++;
            ObjectCreate("S1" + DoubleToStr(objCounter, 0), OBJ_RECTANGLE, 0, Time[i], PP, Time[i], S1); 
            ObjectSet   ("S1" + DoubleToStr(objCounter, 0), OBJPROP_COLOR, MediumSlateBlue); objCounter++;
            
            ObjectCreate("R2" + DoubleToStr(objCounter, 0), OBJ_RECTANGLE, 0, Time[i], R3, Time[i], R2); 
            ObjectSet   ("R2" + DoubleToStr(objCounter, 0), OBJPROP_COLOR, Crimson); objCounter++;
            ObjectCreate("S2" + DoubleToStr(objCounter, 0), OBJ_RECTANGLE, 0, Time[i], S3, Time[i], S2); 
            ObjectSet   ("S2" + DoubleToStr(objCounter, 0), OBJPROP_COLOR, Crimson); objCounter++;
            
            ObjectCreate("R3" + DoubleToStr(objCounter, 0), OBJ_RECTANGLE, 0, Time[i], R1, Time[i], R3); 
            ObjectSet   ("R3" + DoubleToStr(objCounter, 0), OBJPROP_COLOR, Blue); objCounter++;
            ObjectCreate("S3" + DoubleToStr(objCounter, 0), OBJ_RECTANGLE, 0, Time[i], S1, Time[i], S3); 
            ObjectSet   ("S3" + DoubleToStr(objCounter, 0), OBJPROP_COLOR, Blue); objCounter++;
			}
			
			// Reset alert status
         alarm1sent = false; alarm2sent = false; alarm3sent = false;
         alarm4sent = false; alarm5sent = false; alarm6sent = false;			
			
      }
      
      if(colored)
	   {
	      ObjectMove("R1" + DoubleToStr(objCounter - 6, 0), 1, Time[i], R1);
	      ObjectMove("S1" + DoubleToStr(objCounter - 5, 0), 1, Time[i], S1);
         ObjectMove("R2" + DoubleToStr(objCounter - 4, 0), 1, Time[i], R2);
	      ObjectMove("S2" + DoubleToStr(objCounter - 3, 0), 1, Time[i], S2);	      
         ObjectMove("R3" + DoubleToStr(objCounter - 2, 0), 1, Time[i], R3);
	      ObjectMove("S3" + DoubleToStr(objCounter - 1, 0), 1, Time[i], S3);	      
	   }
      
      bufferPP[i] = PP;
      bufferS1[i] = S1;
      bufferR1[i] = R1;
      bufferS2[i] = S2;
      bufferR2[i] = R2;
      bufferS3[i] = S3;
      bufferR3[i] = R3;      
      
      if(email && Time[i] > startTime)
      {
         if(notify1)
         {
            if((Close[i] - R1) * (Close[i + 1] - R1) < 0 && !alarm1sent) sendEmail(1, i); 
            if((Close[i] - S1) * (Close[i + 1] - S1) < 0 && !alarm2sent) sendEmail(2, i); 
         }  
         
         if(notify2)
         {
            if((Close[i] - R2) * (Close[i + 1] - R2) < 0 && !alarm3sent) sendEmail(3, i);  
            if((Close[i] - S2) * (Close[i + 1] - S2) < 0 && !alarm4sent) sendEmail(4, i); 
         }  
         
         if(notify3)
         {
            if((Close[i] - R3) * (Close[i + 1] - R3) < 0 && !alarm5sent) sendEmail(5, i); 
            if((Close[i] - S3) * (Close[i + 1] - S3) < 0 && !alarm6sent) sendEmail(6, i); 
         }           
      }
   }

   return(0);
}
//+------------------------------------------------------------------+
//| Custom indicator sendmail function                               |
//+------------------------------------------------------------------+
int sendEmail(int alarm, int i)
{
   string text;
            
   switch(alarm)
   {
      case 1: text = "\nClose price cross R1 at "; alarm1sent = true; break;
      case 2: text = "\nClose price cross S1 at "; alarm2sent = true; break;
      case 3: text = "\nClose price cross R2 at "; alarm3sent = true; break;
      case 4: text = "\nClose price cross S2 at "; alarm4sent = true; break;
      case 5: text = "\nClose price cross R3 at "; alarm5sent = true; break;
      case 6: text = "\nClose price cross S3 at "; alarm6sent = true; 
   }

   SendMail(emailSubject, emailBody + text + DoubleToStr(Close[i], Digits));
}
//+------------------------------------------------------------------+